==> poker-0.2.1.tar.gz: lib/src/evaluators/exhaustive_evaluator.dart <==
import "dart:collection";
import "package:poker/poker.dart";
import "./evaluator.dart";

class ExhaustiveEvaluator
    with IterableMixin<EvaluationResult>
    implements Evaluator {
  ExhaustiveEvaluator({
    required this.communityCards,
    required this.players,
  });

  final CardSet communityCards;

  final List<HandRange> players;

  Iterator<EvaluationResult> get iterator =>
      ExhaustiveEvaluationIterator(original: this);
}

class ExhaustiveEvaluationIterator implements Iterator<EvaluationResult> {
  ExhaustiveEvaluationIterator({required ExhaustiveEvaluator original})
      : _playerCardPairs = original.players.map((p) => p.toList()).toList(),
        _stack = [
          _ExhaustiveEvaluationNode(
            deck: CardSet.full.difference(original.communityCards),
            playerCardPairs: [],
            communityCards: original.communityCards,
          ),
        ],
        _lastResult = null;

  final List<List<CardPair>> _playerCardPairs;

  final List<_ExhaustiveEvaluationNode> _stack;

  EvaluationResult? _lastResult;

  @override
  EvaluationResult get current => _lastResult!;

  @override
  bool moveNext() {
    if (_stack.isEmpty) return false;

    final node = _stack.removeLast();

    if (node.playerCardPairs.length < _playerCardPairs.length) {
      final nextPlayerIndex = node.playerCardPairs.length;

      for (final playerCardPair in _playerCardPairs[nextPlayerIndex]) {
        if (!node.deck.containsAll(playerCardPair)) continue;

        _stack.add(_ExhaustiveEvaluationNode(
          deck: node.deck.difference(playerCardPair),
          playerCardPairs: [...node.playerCardPairs, playerCardPair],
          communityCards: node.communityCards,
        ));
      }

      // continue;
      return moveNext();
    }

    if (node.communityCards.length < 5) {
      for (final card in node.deck) {
        _stack.add(_ExhaustiveEvaluationNode(
          deck: node.deck.difference(CardSet({card})),
          playerCardPairs: node.playerCardPairs,
          communityCards: node.communityCards.union(CardSet({card})),
        ));
      }

      // continue;
      return moveNext();
    }

    final matchup = Matchup.showdown(
      communityCards: node.communityCards,
      playerCardPairs: node.playerCardPairs,
    );

    _lastResult = EvaluationResult.fromMatchup(matchup);

    return _lastResult != null;
  }
}

class _ExhaustiveEvaluationNode {
  _ExhaustiveEvaluationNode({
    required this.deck,
    required this.playerCardPairs,
    required this.communityCards,
  });

  final CardSet deck;
  final List<CardSet> playerCardPairs;
  final CardSet communityCards;
}

==> poker-0.2.5.tar.gz: lib/src/evaluators/exhaustive_evaluator.dart <==
import "dart:collection";
import "package:poker/poker.dart";
import "./evaluator.dart";

class ExhaustiveEvaluator
    with IterableMixin<EvaluationResult>
    implements Evaluator {
  ExhaustiveEvaluator({
    required this.communityCards,
    required this.players,
  });

  final CardSet communityCards;

  final List<HandRange> players;

  Iterator<EvaluationResult> get iterator =>
      ExhaustiveEvaluationIterator(original: this);
}

class ExhaustiveEvaluationIterator implements Iterator<EvaluationResult> {
  ExhaustiveEvaluationIterator({required ExhaustiveEvaluator original})
      : _playerCardPairs = original.players.map((p) => p.toList()).toList(),
        _stack = [
          _ExhaustiveEvaluationNode(
            deck: CardSet.full.difference(original.communityCards),
            playerCardPairs: [],
            communityCards: original.communityCards,
          ),
        ],
        _lastResult = null;

  final List<List<CardPair>> _playerCardPairs;

  final List<_ExhaustiveEvaluationNode> _stack;

  EvaluationResult? _lastResult;

  @override
  EvaluationResult get current => _lastResult!;

  @override
  bool moveNext() {
    if (_stack.isEmpty) return false;

    final node = _stack.removeLast();

    if (node.playerCardPairs.length < _playerCardPairs.length) {
      final nextPlayerIndex = node.playerCardPairs.length;

      for (final playerCardPair in _playerCardPairs[nextPlayerIndex]) {
        if (!node.deck.containsAll(playerCardPair)) continue;

        _stack.add(_ExhaustiveEvaluationNode(
          deck: node.deck.difference(playerCardPair),
          playerCardPairs: [...node.playerCardPairs, playerCardPair],
          communityCards: node.communityCards,
        ));
      }

      // continue;
      return moveNext();
    }

    if (node.communityCards.length < 5) {
      for (final card in node.deck) {
        _stack.add(_ExhaustiveEvaluationNode(
          deck: node.deck.difference(CardSet.single(card)),
          playerCardPairs: node.playerCardPairs,
          communityCards: node.communityCards.union(CardSet.single(card)),
        ));
      }

      // continue;
      return moveNext();
    }

    final matchup = Matchup.showdown(
      communityCards: node.communityCards,
      playerCardPairs: node.playerCardPairs,
    );

    _lastResult = EvaluationResult.fromMatchup(matchup);

    return _lastResult != null;
  }
}

class _ExhaustiveEvaluationNode {
  _ExhaustiveEvaluationNode({
    required this.deck,
    required this.playerCardPairs,
    required this.communityCards,
  });

  final CardSet deck;
  final List<CardSet> playerCardPairs;
  final CardSet communityCards;
}
